"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.NetworkGenerator = exports.NetworkSettingsConfig = void 0;
const logger_1 = require("../../../lib/logger");
const network_signals_filter_1 = require("./network-signals-filter");
const types_1 = require("../../../types");
const network_interceptor_1 = require("./network-interceptor");
const helpers_1 = require("./helpers");
class NetworkSettingsConfig {
    constructor({ networkSignalsAllowList, networkSignalsDisallowList, networkSignalsAllowSameDomain, }) {
        this.networkSignalsAllowSameDomain = networkSignalsAllowSameDomain ?? true;
        this.networkSignalsFilterList = new network_signals_filter_1.NetworkSignalsFilterList(networkSignalsAllowList, networkSignalsDisallowList);
    }
}
exports.NetworkSettingsConfig = NetworkSettingsConfig;
class NetworkGenerator {
    constructor(settings) {
        this.id = 'network';
        this.interceptor = new network_interceptor_1.NetworkInterceptor();
        /* List of all signal request IDs that have been emitted */
        this.emittedRequestIds = [];
        this.filter = new network_signals_filter_1.NetworkSignalsFilter(settings);
    }
    register(emitter) {
        const createMetadata = () => ({
            filters: this.filter.settings.networkSignalsFilterList.getRegexes(),
        });
        const handleRequest = (rq) => {
            if (!(0, helpers_1.containsJSONContentType)(rq.headers)) {
                return;
            }
            if (!rq.url || !this.filter.isAllowed(rq.url)) {
                return;
            }
            this.emittedRequestIds.push(rq.id);
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'request',
                url: rq.url,
                method: rq.method || 'GET',
                data: rq.body ? JSON.parse(rq.body.toString()) : null,
            }, createMetadata()));
        };
        const handleResponse = async (rs) => {
            const isSuccessWithNonJSONResponse = rs.ok &&
                rs.responseType !== 'json' &&
                !(0, helpers_1.containsJSONContentType)(rs.headers);
            const isErrorButRequestNeverEmittted = !rs.ok && !this.emittedRequestIds.includes(rs.req.id);
            if (isSuccessWithNonJSONResponse || isErrorButRequestNeverEmittted) {
                return;
            }
            const url = rs.url;
            if (!url || !this.filter.isAllowed(url)) {
                return;
            }
            const data = await rs.body();
            emitter.emit((0, types_1.createNetworkSignal)({
                action: 'response',
                url,
                data: data,
                ok: rs.ok,
                status: rs.status,
            }, createMetadata()));
        };
        this.interceptor.addInterceptors(handleRequest, handleResponse);
        return () => {
            this.interceptor.cleanup();
            logger_1.logger.debug('Removing fetch interceptor');
        };
    }
}
exports.NetworkGenerator = NetworkGenerator;
//# sourceMappingURL=index.js.map